class CollectionsController < ApplicationController
  before_action :set_collection, only: [:show, :update, :destroy, :like, :change_privacy]
  before_action :set_collections, only: [:list, :add_learning_object, :remove_learning_object]
  before_action :authenticate_user!, only: [:update, :destroy, :like, :list, :me, :add_learning_object, :remove_learning_object, :change_privacy]

  # GET /collections
  # GET /collections.json
  def index
    respond_to do |format|
      # for HTML page, returns institutional collections
      format.html do
        context = Collections::InstitutionsContext.new(institution_repository.all)
        @collections = collection_repository.all context
      end

      # returns all collection of logged user as JSON for UI actions
      if user_signed_in?
        format.json do
          @collections = current_user.collections
        end
      end
    end
  end

  # GET /collections/1
  # GET /collections/1.json
  def show
    if @collection.class == User
      # bookmark
      @user = @collection
      @collection = collection_repository.bookmarks(@user)
      @own = true
    else
      check_collection_privacy! @collection

      @user = @collection.owner
      @own = user_signed_in? ? @collection.owner?(current_user) : false
    end

    render partial: 'list' if params[:list] == "true"
  end

  # POST /collections/1/like
  def like
    collection_repository.like @collection
  end

  # POST /collections
  # POST /collections.json
  def create
    @collection = Collection.new(collection_params)
    @collection.owner = current_user if collection_params[:owner].blank?

    respond_to do |format|
      if @collection.save
        format.html { redirect_to :back, notice: 'Coleção criada com sucesso.' }
      else
        format.html { render :new }
      end
    end
  end

  # PATCH/PUT /collections/1
  # PATCH/PUT /collections/1.json
  def update
    respond_to do |format|
      if collection_repository.update(collection_params)
        format.html { redirect_to @collection, notice: 'Coleção atualizada com sucesso.' }
      else
        format.html { render :edit }
      end
    end
  end

  # DELETE /collections/1
  # DELETE /collections/1.json
  def destroy
    collection_repository.destroy @collection

    respond_to do |format|
      format.html { redirect_to me_users_path, notice: 'Coleção excluida com sucesso.' }
    end
  end

  def list
    @collection = @collections.first

    # list all
    @collection = nil if @collection == 'all'

    @collections = collection_repository.all(Collections::UserContext.new(current_user))
    @collections.select! { |c| c.id != @collection.id } unless @collection.blank?

    unless params[:type].blank?
      @type = params[:type]

      @send = case @type
      when 'add' then 'Adicionar'
      when 'copy' then 'Copiar'
      when 'move' then 'Mover'
      else 'Enviar'
      end

      @title = (@send == 'Enviar') ? 'Coleções' : @send + '  objeto(s) às coleções'
    end

    render layout: false
  end

  def me
    @new_collection = Collection.new

    @publishers = publishers_for current_user

    @groups = [
        CollectionsGroup.new(title: 'Coleções Automáticas',
                             collections: [current_user.bookmarks]),
        CollectionsGroup.new(title: 'Coleções Adicionadas',
                             collections: current_user.collections)
    ]
  end

  # POST /collections/1/learning_object
  def add_learning_object
    @collections.each do |collection|
      if collection.owner?(current_user)
        @learning_objects.each do |learning_object|
          collection.add learning_object
        end

        collection_repository.save_learning_objects(collection)
      end
    end

    if request.xhr?
      render json: {status: true}
    end
  end

  # DELETE /collections/1/learning_object
  def remove_learning_object
    @collections.each do |collection|
      if collection.owner?(current_user)
        @learning_objects.each do |learning_object|
          collection.remove learning_object
        end

        collection_repository.save_learning_objects(collection)
      end
    end

    if request.xhr?
      render json: {status: true}
    end
  end

  # change collection privacy
  def change_privacy
    collection_repository.change_privacy(@collection, params[:privacy])

    if request.xhr?
      render json: {status: true}
    end
  end

  private
  def check_collection_privacy!(collection)
    if collection.privacy == "private"
      redirect_to :root, notice: 'Está é uma coleção privada.' unless collection.owner?(current_user)
    end
  end

  def set_collection
    @collection = Collection.find params[:id]
  end

  def set_collections
    if params[:id] == "all" || params[:id].blank?
      @collections = ['all']
    else
      @collections = (params[:id].class == String) ? [collection_repository.find(params[:id])] : params[:id].map{|id| collection_repository.find id}
    end

    unless params[:learning_objects_ids].blank?
      @learning_objects = []

      params[:learning_objects_ids].split(',').each do |id|
        object = learning_object_repository.find id
        @learning_objects << object unless object.blank?
      end
    end
  end

  def publishers_for(user)
    user_new = Institution.new()
    user_new.id = user.rid
    user_new.name = user.name

    [user_new] + user.institutions
  end

  # Never trust parameters from the scary internet, only allow the white list through.
  def collection_params
    params.require(:collection).permit(:name, :description, :owner, learning_objects: [])
  end

end
